// Questions to huegon@uv.es, October 2016, Target Systemelektronik Praktikum
		 

#ifndef PulseSurfer_h
#define PulseSurfer_h PulseSurfer_h

#include "RQ_OBJECT.h"
#include "TGFrame.h"
#include <fstream>

//forward declarations
class TextedHSlider;
class TGFileDialog;
class TRootEmbeddedCanvas;
class TGWindow;
class TGHSlider;
class TGNumberEntry;
class TGCheckButton;
class TGStatusBar;
class TGPictureButton;
class TGTextEntry;
class TGComboBox;
class TTimer;
class TH1;
class TH2;

struct FileFormat///< Binary format type and waveform related parameters supported file types for pulse viewing
{	
	UShort_t samplesPerPulse;		///< number of samples per pulse
	UShort_t preTrigger;			///< Position of trigger point, in sample points. For baseline calculation
	Short_t overflow;				///< Overflow bin
	UShort_t bytesPerSample;		///< 2 bytes (int16) are used per sample point. Other things are not supported
	FileFormat() : samplesPerPulse(0), preTrigger(0), overflow(32767), bytesPerSample(2) {}///< Default constructor
};

/**
 * \brief GUI function visualising pulse waveforms interactively
 * \note Next - previous pulse with mouse wheel scroll down / up or pressing n / b. Or with command line: ReadNextPulse() / ReadPreviousPulse()
 * \note Go to event left click mouse or pressing g. Or with command line: ReadPulse(i)
 */
class PulseSurfer : public TGMainFrame
{
	RQ_OBJECT("PulseSurfer")	///< for classes not derived from TQObject, RQ_OBJECT macro should be used
	
	public:
		
		/**
		 * \brief Default class constructor
		 * \param baseDir the optional path to the directory where binary files are stored, for easier searching later with file browser
		 */		
		PulseSurfer(const TString baseDir="");
		
		~PulseSurfer();///< class destructor
		
		/**
		 * \brief Function to open file stream with given binary format
		 * \param fileName the name of the raw file to analyze
		 * \param format the file type parameters
		 * \return true if successfully opened, false otherwise
		 */		
		Bool_t Open(const TString fileName, const FileFormat format);
		
		/**
		 * \brief Function to open file stream with given binary format
		 * \param fileName the name of the raw file to analyze
		 * \param samplesPerPulse number of samples per pulse
		 * \param preTrigger Position of trigger point, in sample points. For baseline calculation
		 * \param overflow Overflow bin
		 * \param bytesPerSample 2 bytes (int16) are used per sample point. Other things are not supported
		 * \return true if successfully opened, false otherwise
		 * \note alias for other Open function 
		 */		
		Bool_t Open(const TString fileName, const UShort_t samplesPerPulse, const UShort_t preTrigger, const Short_t overflow=32767, const UShort_t bytesPerSample=2);
		
		void OpenFileBrowser();	///< Function to open file browser prompt
		
		/**
		 * \brief Tries to open file after filename and parameters set from GUI
		 * \return true if successfully opened, false otherwise
		 */
		Bool_t OpenFromGUI();
		/**
		 * \brief Check the integrity of an input binary file
		 * \param fileName the name of the file
		 * \param bytesPerEvent the number of bytes each event has
		 * \return the number of events, or 0 if file is corrupt
		 */
		static ULong64_t CheckFile(const TString fileName, const UShort_t bytesPerPulse);
		
		/**
		 * \brief It reads the next pulse in the stream
		 * \return true or false depending on success/failure
		 */
		Bool_t ReadNextPulse();
		
		/**
		 * \brief It reads the previous pulse in the stream
		 * \return true or false depending on success/failure
		 */
		Bool_t ReadPreviousPulse();
		
		/**
		 * \brief It reads a certain pulse in the stream
		 * \param event the pulse number
		 * \param emit if true, it emits a signal for updating parent canvas when called through GUI slider
		 * \return true or false depending on success/failure
		 */
		Bool_t ReadPulse(const ULong64_t event, const Bool_t emit);//*SIGNAL*
	
		/**
		 * \brief It reads a certain pulse in the stream, from slider
		 * \param event the pulse number
		 * \return true or false depending on success/failure
		 */
		Bool_t ReadPulseEmit(const ULong64_t event);
	
		/**
		 * \brief Enables or disables automatic baseline subtraction
		 * \param subtract true if enable, false if disable
		 */
		void SubtractBaseline(const Bool_t subtract=kTRUE);
		
		/**
		 * \brief Sets the persistency, i.e. option to accumulate traces in 2D histogram
		 * \param persist true if enable, false if disable
		 */
		void SetPersistency(const Bool_t persist = kTRUE);
		
		/**
		 * \brief Handles key press events for going forward or backward
		 * \param event the type of event
		 * \param px the key character
		 * \param py the key symbol/address
		 * \param sel the object selected where event happens
		 * \note It has to be public!
		 */
		void CaptureEvent(Int_t event, Int_t px, Int_t py, TObject *sel);
		
		/**
		 * \brief Flushes the combo box samplesPerPulse value to the internal variable
		 * \return false if out of range, true otherwise
		 */
		Bool_t UpdateSamplesPerPulse();
		
		/**
		 * \brief Flushes the combo box preTrigger value to the internal variable
		 * \return false if out of range, true otherwise
		 */
		Bool_t UpdatePreTrigger();
		
		/**
		 * \brief Flushes the combo box overflow value to the internal variable
		 * \return false if out of range, true otherwise
		 */
		Bool_t UpdateOverflow();
		
		void Rewind();///< Starts the backward autoplay
		void Play();///< Starts the forward autoplay
		void Stop();///< Stops the autoplay
		Bool_t TimerUpdate();///< Plays next step \return true or false depending on success / failure (on reading next pulse)

	protected:
		void RecreateHistogram();	///< (Re)creates the pulse histogram
		void CloseWindow();			///< Closes the GUI. Automatically called	
				
	private:
		TString fBaseDir;					///< the base directory for file searching with browser
		TString fFileName;					///< name of the file to analyse
		FileFormat fFormat;					///< file format
		UShort_t fBytesPerPulse;			///< number of bytes per pulse
		ULong64_t fTotalEvents;				///< number of events within file
		ULong64_t fCurrentEvent;			///< current event to analyse
		Bool_t fSubtractBaseline;			///< apply baseline subtraction
		Bool_t fPersistency;				///< option to accumulate traces for 2D histogram
		
		std::ifstream fFile;				///< File reader		
		UShort_t fFrequence[65536];			///< array for storing an ADC histogram of each pulse
		
		Char_t* fRawWaveform;				///< Stores the raw waveform stream
		Short_t* fADC;						///< It assumes the casted waveform, 2 bytes, signed int16. Class should be changed or templated if anything else is wanted
		
		TRootEmbeddedCanvas* fCanvas;		///< The main canvas
		TH1* fHist;							///< The pulse profile
		TH2* fHist2D;						///< The 2D pulse profile accumulation
		
		TGStatusBar*   fStatusBar;			///< Event status bar
		
		TGHorizontalFrame* fFileFrame;		///< Frame containing the filename, and related format parameters
		TGPictureButton *fOpenIcon;			///< File open icon
		TGFileDialog*  fFileDialog;			///< Prompt dialog for file selection
		TGTextEntry* fFileEntry;			///< Text entry for file selection
		TGComboBox* fSamplesPerPulseBox;	///< Combo box to select samples per pulse
		TGComboBox* fPreTriggerBox;			///< Combo box to select samples per pulse
		TGComboBox* fOverflowBox;			///< Combo box to select samples per pulse
		TGComboBox* fBytesPerSampleBox;		///< Combo box to select samples per pulse
		
		TGHorizontalFrame* fSettingsFrame;	///< Frame containing the event slider and analysis parameters
		TGCheckButton* fSubtractButton;		///< Check button for baseline subtraction
		TGCheckButton* fPersistencyButton;	///< Check button for 2D trace accumulation
		TextedHSlider* fEventSlider;		///< Slider for moving across pulses
		
		TGPictureButton *fForwardIcon;		///< Fast forward player icon
		TGPictureButton *fRewindIcon;		///< Fast rewind player icon
		TGPictureButton *fStopIcon;			///< Stop player icon
		TTimer* fTimer;						///< Autoplay timer

	#if ROOT_VERSION_CODE < ROOT_VERSION(6,0,0)//Just needed for ROOT v5
	ClassDef(PulseSurfer,1);///needed as it derives from TObject
	#endif
};

/**
 * \brief This class is a modified GUI slider with associated number entry, so that you can either choose to move slider or write the number.
 */
class TextedHSlider
{
	public:
		/**
		 * \brief Constructor
		 * \param p the parent GUI window where slider will be located
		 * \param w width
		 * \param type the slider type
		 * \param id a slider-specific code
		 * \param options layout options for the frame
		 * \param back the background color
		 * \param start min allowed value of the calibrating line slider. 
		 * \param end max allowed value of the calibrating line slider. 
		 * \param position the initial position of the calibrating line slider.
		 * \param isReal true if the calibrating line position has non-integer values, false otherwise.
		 */
		TextedHSlider(const TGWindow* p, UInt_t w, UInt_t type, Int_t id, UInt_t options, Pixel_t back, const Double_t start, const Double_t end, const Double_t position, Bool_t isReal);
		//~ TextedHSlider(const TGWindow* p = 0, UInt_t w = 40, UInt_t type = kSlider1|kScaleBoth, Int_t id = -1, UInt_t options = kHorizontalFrame, Pixel_t back = TGHSlider::GetDefaultFrameBackground(), const Double_t start = 0, const Double_t end = 1, const Double_t position = 0, Bool_t isReal = false);

		virtual ~TextedHSlider();///< Destructor
		
		TGHSlider* GetSlider();///< Gets the horizontal slider (internal class member) \return TGHSlider
		TGNumberEntry* GetNumberEntry();///< Gets the associated number entry of the class \return TGNumberEntry
		void MoveSlider();///< Moves the (integer value) slider after the number entry was modified via the GUI.
		void MoveRealThreeSlider();///< Moves the (decimal value) slider after the number entry was modified via the GUI.
		void MoveNumber(Int_t pos);///< Moves the (integer value) number entry after the slider position was modified via the GUI. \param pos the slider position to be moved to
		void MoveRealThreeNumber(Int_t pos);///< Moves the (decimal value) number entry after the slider position was modified via the GUI. \param pos the slider position that was set in the GUI.
		
	private:
		TGHSlider* fSlider;	///< internal horizontal slider
		TGNumberEntry* fNumber;	///< internal number entry associated to the horizontal slider
};

#endif
