/**
 * \brief Run as root -l test_gui.cpp+    or
 *               root -l test_gui.cpp+(1) or
 * 				 root -l test_gui.cpp+(\"F500file-00001.bin\",8192,2000)
 * \note Tested with ROOT v5.34 on Windows and Linux, and with ROOT6 on Linux.
 * \note If run with ROOT5, run instead test_gui_v5.cpp
 * \note Call before source /opt/root/bin/thisroot.sh
 * \note Some GUI bugs with TGComboBox and TGHSlider are just corrected in ROOT6 v6.08 
 * \see https://root.cern.ch/phpBB3/viewtopic.php?f=3&t=22554 https://sft.its.cern.ch/jira/browse/ROOT-8406
 * \note https://root.cern.ch/phpBB3/viewtopic.php?f=3&t=22434
 */

#include "PulseSurfer.cpp"
PulseSurfer* ps = NULL;//global variable pointer

/**
 * \brief Function to open file stream with given binary format through GUI
 * \param fileName the name of the raw file to analyze
 * \param samplesPerPulse number of samples per pulse
 * \param preTrigger Position of trigger point, in sample points. For baseline calculation
 * \param overflow Overflow bin
 * \param bytesPerSample 2 bytes (int16) are used per sample point. Other things are not supported
 * \param subtractBaseline automatic baseline subtraction
 * \param persist set 2D persistency accumulation
 * \param xmin zoom min x value
 * \param xmax zoom max x value
 * \param ymin zoom min y value
 * \param ymax zoom max y value
 */		
void test_gui(const TString fileName, const UShort_t samplesPerPulse, const UShort_t preTrigger, const Short_t overflow=32767, const UShort_t bytesPerSample=2, const Bool_t subtractBaseline=kTRUE, const Bool_t persist=kFALSE, const Int_t xmin=0, const Int_t xmax=8192, const Int_t ymin=0, const Int_t ymax=16000)
{
	ps = new PulseSurfer();
	ps->SubtractBaseline(subtractBaseline);//enables auto baseline subtraction
	ps->Open(fileName,samplesPerPulse,preTrigger,overflow,bytesPerSample);
	ps->SetPersistency(persist);//enables auto 2D trace accumulation
	
	TCanvas* c = (TCanvas*)gROOT->FindObject(kCanvasName);
	if(!c) return;
	TH1* h = (TH1*)gROOT->FindObject(kHistName);
	if(!h) return;
	h->GetXaxis()->SetRangeUser(xmin,xmax);
	h->GetYaxis()->SetRangeUser(ymin,ymax);
}

/**
 * \brief Function to test the GUI
 * \param id some id parameter
 */
void test_gui(const Int_t id = -1)
{
	// Check if PulseSurfer is already loaded
	if (!TClass::GetDict("PulseSurfer")) {
		//~ gROOT->ProcessLine(".L PulseSurfer.cpp+");
	}
	
	cout << "Analyze your pulses interactively. Move your cursor inside the canvas region and scroll your wheel. Or press n or b for next/back pulse, or use slider." << endl;
	cout << "You can open a new file with ps->Open(\"a.bin\",4096,613) or via the GUI, or within the script." << endl;
	cout << "You can fix your axes zoom clicking on the x and y axis, SetRangeUser. Or within the script." << endl;
	cout << "Note that you need GLX installed for faster drawing. Check it running glxgears in command line, and root -b." << endl;
	
	//Default file formats
	FileFormat u100;
	u100.samplesPerPulse=4096;
	u100.preTrigger=613;

	FileFormat f500_8k;
	f500_8k.samplesPerPulse=8192;
	f500_8k.preTrigger=2000;
	f500_8k.overflow=16383;
	
	//Implement here your own one if needed
	
	if(id==-1)
	{
		ps = new PulseSurfer();
	}
	else if(id==0)
	{
		ps = new PulseSurfer();
		ps->SubtractBaseline();//enables auto baseline subtraction
		ps->Open("F500file-00001.bin",f500_8k);
		
		TCanvas* c = (TCanvas*)gROOT->FindObject(kCanvasName);
		if(!c) return;
		c->SetLogz();
		TH1* h = (TH1*)gROOT->FindObject(kHistName);
		if(!h) return;
		h->GetXaxis()->SetRangeUser(1800,6000);
		h->GetYaxis()->SetRangeUser(0,4000);
	}
	else if(id==1)
	{
		ps = new PulseSurfer();
		ps->SubtractBaseline();//enables auto baseline subtraction
		ps->Open("u100-p0017.local_fwk207_2016-05-10_13.34.48+0200.din",u100);
		
		TCanvas* c = (TCanvas*)gROOT->FindObject(kCanvasName);
		if(!c) return;
		c->SetLogz();
		TH1* h = (TH1*)gROOT->FindObject(kHistName);
		if(!h) return;
		h->GetXaxis()->SetRangeUser(600,700);
		h->GetYaxis()->SetRangeUser(0,1000);
	}
	//add here more files
	
	return;
}
